/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/
#include "stdafx.h"
#include "SAScrOper.h"
#include "SAScrDefs.h"
#include "SAScrUtils.h"

//////////////////////////////////////////////////////////////////////////

CSAScrOp::CSAScrOp() 
{
}

//////////////////////////////////////////////////////////////////////////

CSAScrOp::CSAScrOp(int i)                  
{ 
   m_text.Format(_T( "%d" ),i); m_type = eOperand;
}

//////////////////////////////////////////////////////////////////////////

CSAScrOp::CSAScrOp(const CSAString &s)    
{ 
   m_text = s; 
   m_type = eOperand;
}

//////////////////////////////////////////////////////////////////////////

CSAScrOp::CSAScrOp(const eParseContext c)  
{ 
   m_text.Format(_T( "%d" ), (int)c); 
   m_type = eOperand;
}

//////////////////////////////////////////////////////////////////////////

CSAScrOp::CSAScrOp(const CSAScrOperator inOp)  
{ 
   m_operator = inOp; 
   m_type = eOperator;
}

//////////////////////////////////////////////////////////////////////////

CSAScrOp::CSAScrOp(const CSAScrVariable &s) : m_operator(s) 
{
   m_type = eVar;
}

//////////////////////////////////////////////////////////////////////////

bool operator < (const CSAScrOp &a, const CSAScrOp &b)
{
   return a.m_text < b.m_text;
}

//////////////////////////////////////////////////////////////////////

bool operator == (const CSAScrOp &a, const CSAScrOp &b)
{
   return (a.m_text == b.m_text);
}

//////////////////////////////////////////////////////////////////////

CSAScrOperator::CSAScrOperator()
{
   m_pFnOperator = NULL;      
   m_iParamCount = 0;    
	m_bMacroFunction = false;
}

//////////////////////////////////////////////////////////////////////////

CSAScrOperator::CSAScrOperator(const CSAScrVariable &_v)
{
   m_vs = _v;
}

//////////////////////////////////////////////////////////////////////////

CSAScrOperator::CSAScrOperator(const TCHAR	* pName, 
               pVarEvalFunc pFunc, 
               int iPCount, 
               const TCHAR	*   pParams, 
               const TCHAR	*   pFuncString, 
               const TCHAR	*   pDesc,
					bool bMacro)
{
   m_csOperatorName = pName;
   m_pFnOperator = pFunc;
   m_iParamCount = iPCount;
   m_csParamName = pParams;
   m_csFunctionString = pFuncString;
   m_csDescription = pDesc;
	m_bMacroFunction = bMacro;
}

//////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
void CSAScrOperator::Dump() const
{
   SAScr::SAScrTrace(_T( "* %s (%s) : " ), (const TCHAR	*)m_csOperatorName, (const TCHAR	*)m_csParamName);
   if ((const TCHAR	*)m_csDescription!=NULL)
   {
      SAScr::SAScrTrace(_T( "%s " ), (const TCHAR	*)m_csDescription);
   }

   if ((const TCHAR	*)m_csFunctionString!=NULL)
   {
      SAScr::SAScrTrace(_T( "%s " ), (const TCHAR	*)m_csFunctionString);
   }

   SAScr::SAScrTrace(_T( "\n" ));
}
#endif

//////////////////////////////////////////////////////////////////////////
#ifdef _DEBUG
void CSAScrOperatorMap::Dump() const
{
   // clear user macros
   const_iterator theIterator;
   for (theIterator = begin(); theIterator != end(); theIterator++)
   {
      (*theIterator).second.Dump();
   }
}
#endif


//////////////////////////////////////////////////////////////////////////

void CSAScrOperatorMap::CleanUp()
{
   // clear user macros
   iterator theIterator;
   for (theIterator = begin(); theIterator != end(); theIterator++)
   {
      CSAScrOperator pCur = (*theIterator).second;

   }

   clear();
}

//////////////////////////////////////////////////////////////////////////

CSAScrOperatorMap::~CSAScrOperatorMap() 
{
   CleanUp();
}

//////////////////////////////////////////////////////////////////////////

void CSAScrOperatorMap::Add(const CSAScrOperator &oper)
{
   // add the operator
   CSAString csName = oper.m_csOperatorName;

   // store upper case names because maps are case sensitive
   csName.MakeUpper();

   pair<CSAScrOperatorMap::iterator, bool> res = insert(CSAScrOperatorMap::value_type(csName, oper));
}

//////////////////////////////////////////////////////////////////////////

void CSAScrOperatorMap::AddMacros(const CSAMacroMap &userMacroMap)
{
	// add user macros
	CSAMacroMap::const_iterator theIterator;
	for (theIterator = userMacroMap.begin(); theIterator != userMacroMap.end(); theIterator++)
	{
		CSAMacro curMacro = (*theIterator).second;
		
		CSAString csName = curMacro.m_csName;
		csName.MakeUpper();
		
		// parse params
		CSAStringArray csaParams;
		CSAMacro::GetFormalParamArray(curMacro.m_csParams, csaParams);
		int iParamCount = csaParams.GetSize();
		
		CSAScrOperator cur(csName, 
			               NULL,
			               iParamCount,
			               curMacro.m_csParams,
			               curMacro.m_csFunc,
			               _T( "<!--user func-->" ),
			               true);
		
#ifndef SASCR_SILENT
		cur.Dump();
#endif
		
		Add(cur);
	}
}
