/*********************************************************************

   Copyright (C) 2002 Smaller Animals Software, Inc.

   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.

   Permission is granted to anyone to use this software for any purpose,
   including commercial applications, and to alter it and redistribute it
   freely, subject to the following restrictions:

   1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.

   2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.

   3. This notice may not be removed or altered from any source distribution.

   http://www.smalleranimals.com
   smallest@smalleranimals.com

**********************************************************************/
#ifndef VAR_OPERH
#define VAR_OPERH

#include "SAScrVariable.h"
#include "SAScrEvalFunc.h"
#include "SAScrMacros.h"

////////////////////////////////////////////////////////////////
// holds info about an operator
class CSAScrOperator
{
public:
   CSAScrOperator();

   CSAScrOperator(const CSAScrVariable &_v);

   CSAScrOperator(const TCHAR * pName, 
							pVarEvalFunc pFunc, 
							int iPCount, 
							const TCHAR * pParams, 
							const TCHAR * pFuncString, 
							const TCHAR * pDesc,
							bool bMacro = false);

   CSAString	m_csOperatorName;
   pVarEvalFunc   m_pFnOperator;
   int			m_iParamCount;
   CSAString	m_csParamName;
   CSAString	m_csFunctionString;
   CSAString	m_csDescription;
   CSAScrVariable      m_vs;
   
   bool			m_bMacroFunction;

#ifdef _DEBUG
	void Dump() const;
#else
	_inline void Dump() const {}
#endif
};

////////////////////////////////////////////////////////////////

typedef enum eParseContext {eParseCtx1 = 1, eParseCtx2};

////////////////////////////////////////////////////////////////
// operator or operand (or variable)
class CSAScrOp
{
public:
   enum type{eOperator, eOperand, eVar};

   CSAScrOp();
   
   CSAScrOp(int i);
   
   CSAScrOp(const CSAString &s);
   
   CSAScrOp(const eParseContext c);
   
   CSAScrOp(const CSAScrOperator inOp);

   CSAScrOp(const CSAScrVariable &s);
   
   CSAString  m_text;
   type     m_type;
   CSAScrOperator m_operator;
};

////////////////////////////////////////////////////////////////

class CSAScrOperatorMap : public map <CSAString, CSAScrOperator>
{
public:
   ~CSAScrOperatorMap();

   void CleanUp();

   void Add(const CSAScrOperator &);  

   void AddMacros(const CSAMacroMap &userMacroMap);

#ifdef _DEBUG
	void Dump() const;
#else
	_inline void Dump() const {}
#endif
};

////////////////////////////////////////////////////////////////

extern bool operator < (const CSAScrOp &a, const CSAScrOp &b);
extern bool operator == (const CSAScrOp &a, const CSAScrOp &b);

#endif